from .libnonumpy import LibNonumpy


class EMA:
    def __init__(self, lib: LibNonumpy, alpha: float, value: float = 0):
        self._lib = lib

        self._state = self._lib.new('struct nonumpy_ema_state_t')
        self._state.alpha = alpha
        self._state.value = value

        self._init_func = getattr(self._lib, f'nonumpy_ema_init')
        self._run_func = getattr(self._lib, f'nonumpy_ema_run')

        self._init_func(self._state)

    def run(self, value: float) -> float:
        return self._run_func(self._state, value)



from enum import IntEnum
from .libnonumpy import LibNonumpy


class FilterNameEnum(IntEnum):
    Butterworth = 1


class FilterTypeEnum(IntEnum):
    LowPass = 101,
    HighPass = 102,
    BandPass = 103,
    BandStop = 104


class Filter:
    _tokens = {
        FilterNameEnum.Butterworth: 'bw',
        FilterTypeEnum.LowPass: 'lp',
        FilterTypeEnum.HighPass: 'hp',
    }

    def __init__(self, lib: LibNonumpy,
                 name: FilterNameEnum,
                 type: FilterTypeEnum,
                 sample_rate_hz: float,
                 order: int,
                 cutoff_high_hz: float = None,
                 cutoff_low_hz: float = None):

        self._lib = lib
        self._name = name
        self._type = type
        self._order = order
        self._sample_rate_hz = sample_rate_hz
        self._cutoff_high_hz = cutoff_high_hz
        self._cutoff_low_hz = cutoff_low_hz

        self.init_func = None
        self.run_func = None

        self._init_impl()

    def _init_impl(self) -> bool:
        key = f'{Filter._tokens[self._name]}_{Filter._tokens[self._type]}_{self._order}'

        self._filter_state = self._lib.new(
            f'struct nonumpy_filter_{key}_state_t')

        self.init_func = getattr(self._lib, f'nonumpy_filter_{key}_init')
        self.run_func = getattr(self._lib, f'nonumpy_filter_{key}_run')

        if FilterTypeEnum.LowPass == self._type:
            self.init_func(self._filter_state,
                           self._sample_rate_hz, self._cutoff_low_hz)
        elif FilterTypeEnum.HighPass == self._type:
            self.init_func(self._filter_state,
                           self._sample_rate_hz, self._cutoff_low_hz)

        return True

    def run(self, value: float) -> float:
        return self.run_func(self._filter_state, value)



from .filters import FilterNameEnum, FilterTypeEnum
from .filters import Filter
from .ema import EMA


class Factory:
    def __init__(self, verbose: bool = False, debug: bool = False, path=None):
        # self._verbose = verbose
        # self._debug = debug
        # self._gen_folder = path
        self._lib = Factory._build_lib(verbose=verbose, debug=debug, path=path)

    # @classmethod
    # def CreateFilterLowPassButterworth(cls, sample_rate_hz: float, order: int, cutoff_hz: float):
    #     return Filter(
    #         name=FilterNameEnum.Butterworth,
    #         type=FilterTypeEnum.LowPass,
    #         order=order,
    #         sample_rate_hz=sample_rate_hz,
    #         cutoff_low_hz=cutoff_hz)

    # @classmethod
    # def CreateFilterHighPassButterworth(cls, sample_rate_hz: float, order: int, cutoff_hz: float):
    #     return Filter(
    #         name=FilterNameEnum.Butterworth,
    #         type=FilterTypeEnum.HighPass,
    #         order=order,
    #         sample_rate_hz=sample_rate_hz,
    #         cutoff_low_hz=cutoff_hz)

    # @classmethod
    # def CreateEMA(cls, alpha: float):
    #     return EMA(alpha=alpha)

    def create_FilterLowPassButterworth(self, sample_rate_hz: float, order: int, cutoff_hz: float):
        return Filter(self._lib,
                      name=FilterNameEnum.Butterworth,
                      type=FilterTypeEnum.LowPass,
                      order=order,
                      sample_rate_hz=sample_rate_hz,
                      cutoff_low_hz=cutoff_hz)

    def create_FilterHighPassButterworth(self, sample_rate_hz: float, order: int, cutoff_hz: float):
        return Filter(self._lib,
                      name=FilterNameEnum.Butterworth,
                      type=FilterTypeEnum.HighPass,
                      order=order,
                      sample_rate_hz=sample_rate_hz,
                      cutoff_low_hz=cutoff_hz)

    def create_EMA(self, alpha: float):
        return EMA(self._lib, alpha=alpha)

    @staticmethod
    def _build_lib(verbose: bool, debug: bool, path: str):
        from crelm import Tube

        if verbose:
            print(f'building libnumpy, DEBUG={debug}, gen folder={path}')

        paste = Tube('nonumpy') \
            .verbose(verbose) \
            .set_source_folder_from(__file__) \
            .add_header_files(['nonumpy_ema.h', 'nonumpy_filters.h']) \
            .add_source_files(['nonumpy_ema.c', 'nonumpy_filters.c']) \
            .add_macro('NONUMPY_FLOAT_32') \
            .add_macro_if(debug, 'DEBUG') \
            .squeeze()

        return paste
